package ru.dbotthepony.kstarbound.gl

import org.lwjgl.opengl.GL46.*

// GL_INVALID_ENUM
// GL_INVALID_VALUE
// GL_INVALID_OPERATION
// GL_STACK_OVERFLOW
// GL_STACK_UNDERFLOW
// GL_OUT_OF_MEMORY

sealed class OpenGLError(message: String, val statusCode: Int) : Throwable(message)

class OpenGLUnknownError(statusCode: Int, message: String = "Unknown OpenGL error occured: $statusCode") : OpenGLError(message, statusCode)

class OpenGLInvalidEnumException(message: String = "Invalid enum provided") : OpenGLError(message, GL_INVALID_ENUM)
class OpenGLInvalidValueException(message: String = "Invalid value provided") : OpenGLError(message, GL_INVALID_VALUE)
class OpenGLInvalidOperationException(message: String = "Invalid operation in this context or invalid arguments provided") : OpenGLError(message, GL_INVALID_OPERATION)
class OpenGLStackOverflowException(message: String = "Stack overflow in OpenGL") : OpenGLError(message, GL_STACK_OVERFLOW)
class OpenGLStackUnderflowException(message: String = "Stack underflow in OpenGL") : OpenGLError(message, GL_STACK_UNDERFLOW)
class OpenGLOutOfMemoryException(message: String = "Out of Memory in OpenGL") : OpenGLError(message, GL_OUT_OF_MEMORY)

fun checkForGLError() {
	when (val errorCode = glGetError()) {
		GL_NO_ERROR -> {}
		GL_INVALID_ENUM -> throw OpenGLInvalidEnumException()
		GL_INVALID_VALUE -> throw OpenGLInvalidValueException()
		GL_INVALID_OPERATION -> throw OpenGLInvalidOperationException()
		GL_STACK_OVERFLOW -> throw OpenGLStackOverflowException()
		GL_STACK_UNDERFLOW -> throw OpenGLStackUnderflowException()
		GL_OUT_OF_MEMORY -> throw OpenGLOutOfMemoryException()
		else -> throw OpenGLUnknownError(errorCode)
	}
}
