package ru.dbotthepony.kstarbound.math

class Matrix4fStack {
	private val stack = ArrayDeque<MutableMatrix4f>()

	init {
		stack.add(MutableMatrix4f())
	}

	val last get() = stack.last()

	fun push(matrix4f: MutableMatrix4f = last.copy()): Matrix4fStack {
		stack.add(matrix4f)
		return this
	}

	fun pop(): Matrix4fStack {
		stack.removeLast()
		return this
	}

	fun clear(matrix: MutableMatrix4f = MutableMatrix4f()): Matrix4fStack {
		stack.clear()
		stack.add(matrix)
		return this
	}

	operator fun plus(other: FloatMatrix<*>): Matrix4fStack {
		last.plus(other)
		return this
	}

	operator fun minus(other: FloatMatrix<*>): Matrix4fStack {
		last.minus(other)
		return this
	}

	operator fun times(other: FloatMatrix<*>): Matrix4fStack {
		last.times(other)
		return this
	}

	operator fun plus(other: Float): Matrix4fStack {
		last.plus(other)
		return this
	}

	operator fun minus(other: Float): Matrix4fStack {
		last.minus(other)
		return this
	}

	operator fun times(other: Float): Matrix4fStack {
		last.times(other)
		return this
	}

	fun scale(x: Float = 1f, y: Float = 1f, z: Float = 1f, w: Float = 1f): Matrix4fStack {
		last.scale(x, y, z, w)
		return this
	}

	fun translate(x: Float = 0f, y: Float = 0f, z: Float = 0f): Matrix4fStack {
		last.translate(x, y, z)
		return this
	}

	fun translateWithScale(x: Float = 0f, y: Float = 0f, z: Float = 0f): Matrix4fStack {
		last.translateWithScale(x, y, z)
		return this
	}

	fun translate(vec: Vector3f): Matrix4fStack {
		last.translate(vec)
		return this
	}

	fun translateWithScale(vec: Vector3f): Matrix4fStack {
		last.translateWithScale(vec)
		return this
	}

	fun replace(matrix4f: MutableMatrix4f): Matrix4fStack {
		stack.removeLast()
		stack.add(matrix4f)
		return this
	}
}
