package ru.dbotthepony.kstarbound.world

import ru.dbotthepony.kstarbound.defs.TileDefinition
import ru.dbotthepony.kstarbound.math.Vector2i

class World(val seed: Long = 0L) {
	private val chunkMap = ArrayList<Pair<ChunkPos, Chunk>>()
	private var lastAccessedChunk: Chunk? = null

	fun getChunk(pos: ChunkPos): Chunk? {
		if (lastAccessedChunk?.pos == pos) {
			return lastAccessedChunk
		}

		for ((k, v) in chunkMap) {
			if (k == pos) {
				lastAccessedChunk = v
				return v
			}
		}

		return null
	}

	fun getOrMakeChunk(pos: ChunkPos): Chunk {
		if (lastAccessedChunk?.pos == pos) {
			return lastAccessedChunk!!
		}

		for ((k, v) in chunkMap) {
			if (k == pos) {
				return v
			}
		}

		val chunk = Chunk(this, pos)
		lastAccessedChunk = chunk
		chunkMap.add(pos to chunk)
		return chunk
	}

	fun getForegroundView(pos: ChunkPos): TileChunkView? {
		val get = getChunk(pos) ?: return null

		return TileChunkView(
			center = get.foreground,
			left = getChunk(pos.left())?.foreground,
			top = getChunk(pos.up())?.foreground,
			topLeft = getChunk(pos.up().left())?.foreground,
			topRight = getChunk(pos.up().right())?.foreground,
			right = getChunk(pos.right())?.foreground,
			bottom = getChunk(pos.down())?.foreground,
			bottomLeft = getChunk(pos.down().left())?.foreground,
			bottomRight = getChunk(pos.down().right())?.foreground,
		)
	}

	fun getBackgroundView(pos: ChunkPos): TileChunkView? {
		val get = getChunk(pos) ?: return null

		return TileChunkView(
			center = get.background,
			left = getChunk(pos.left())?.background,
			top = getChunk(pos.up())?.background,
			topLeft = getChunk(pos.up().left())?.background,
			topRight = getChunk(pos.up().right())?.background,
			right = getChunk(pos.right())?.background,
			bottom = getChunk(pos.down())?.background,
			bottomLeft = getChunk(pos.down().left())?.background,
			bottomRight = getChunk(pos.down().right())?.background,
		)
	}

	/**
	 * Считается, что [pos] это абсолютные координаты ТАЙЛА в мире, поэтому они
	 * трансформируются в координаты чанка
	 */
	fun getChunk(pos: Vector2i) = getChunk(ChunkPos(pos))

	/**
	 * Считается, что [pos] это абсолютные координаты ТАЙЛА в мире, поэтому они
	 * трансформируются в координаты чанка
	 */
	fun getOrMakeChunk(pos: Vector2i) = getOrMakeChunk(ChunkPos(pos))

	/**
	 * Считается, что [pos] это абсолютные координаты ТАЙЛА в мире, поэтому они
	 * трансформируются в координаты чанка
	 */
	fun getForegroundView(pos: Vector2i) = getForegroundView(ChunkPos(pos))

	/**
	 * Считается, что [pos] это абсолютные координаты ТАЙЛА в мире, поэтому они
	 * трансформируются в координаты чанка
	 */
	fun getBackgroundView(pos: Vector2i) = getBackgroundView(ChunkPos(pos))

	/**
	 * Считается, что [x] и [y] это абсолютные координаты ЧАНКА в мире, поэтому они
	 * НЕ трансформируются в координаты чанка, а используются напрямую
	 */
	fun getChunk(x: Int, y: Int) = getChunk(ChunkPos(x, y))

	/**
	 * Считается, что [x] и [y] это абсолютные координаты ЧАНКА в мире, поэтому они
	 * НЕ трансформируются в координаты чанка, а используются напрямую
	 */
	fun getOrMakeChunk(x: Int, y: Int) = getOrMakeChunk(ChunkPos(x, y))

	/**
	 * Считается, что [x] и [y] это абсолютные координаты ЧАНКА в мире, поэтому они
	 * НЕ трансформируются в координаты чанка, а используются напрямую
	 */
	fun getForegroundView(x: Int, y: Int) = getForegroundView(ChunkPos(x, y))

	/**
	 * Считается, что [x] и [y] это абсолютные координаты ЧАНКА в мире, поэтому они
	 * НЕ трансформируются в координаты чанка, а используются напрямую
	 */
	fun getBackgroundView(x: Int, y: Int) = getBackgroundView(ChunkPos(x, y))

	fun getTile(pos: Vector2i): ChunkTile? {
		return getChunk(pos)?.foreground?.get(pos.x, pos.y)
	}

	fun setTile(pos: Vector2i, tile: TileDefinition?): Chunk {
		val chunk = getOrMakeChunk(pos)
		chunk.foreground[pos.x and CHUNK_SIZE_FF, pos.y and CHUNK_SIZE_FF] = tile
		return chunk
	}

	fun getBackgroundTile(pos: Vector2i): ChunkTile? {
		return getChunk(pos)?.background?.get(pos.x, pos.y)
	}

	fun setBackgroundTile(pos: Vector2i, tile: TileDefinition?): Chunk {
		val chunk = getOrMakeChunk(pos)
		chunk.background[pos.x and CHUNK_SIZE_FF, pos.y and CHUNK_SIZE_FF] = tile
		return chunk
	}
}
